Class: Tda::Option

Inherits:
Object
  • Object
show all
Includes:
HTTParty
Defined in:
app/models/tda/option.rb

Class Method Summary collapse

Class Method Details

.close_credit_callObject



128
129
# File 'app/models/tda/option.rb', line 128

def self.close_credit_call
end

.close_long_debit_call_spreadObject



130
131
# File 'app/models/tda/option.rb', line 130

def self.close_long_debit_call_spread
end

.close_short_debit_put_spreadObject



132
133
# File 'app/models/tda/option.rb', line 132

def self.close_short_debit_put_spread
end

.create_credit_call(outer:, inner:, q:, price:) ⇒ Object



143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
# File 'app/models/tda/option.rb', line 143

def self.create_credit_call outer:, inner:, q:, price:
  query = {
    orderType: "NET_DEBIT",
    session: "NORMAL",
    price: price,
    duration: "DAY",
    orderStrategyType: "SINGLE",
    orderLegCollection: [
      {
        instruction: "BUY_TO_OPEN",
        quantity: q,
        instrument: {
          symbol: outer.symbol,
          assetType: "OPTION",
        },
      },
      {
        instruction: "SELL_TO_OPEN",
        quantity: q,
        instrument: {
          symbol: inner.symbol,
          assetType: "OPTION",
        },
      },
    ],
  }
  File.write('tmp/query.json', JSON.pretty_generate( query ))
  puts! query, 'query'

  return

  headers = {
    Authorize: "Bearer #{::TD_AMERITRADE[:access_token]}",
  }

  path = "/v1/accounts/#{::TD_AMERITRADE[:accountId]}/orders"
  puts! path, 'path'
  out = self.post path, { query: query, headers: headers }
  timestamp = DateTime.parse out.headers['date']
  out = out.parsed_response.deep_symbolize_keys
  puts! out, 'created credit call?'
end

.create_long_debit_call_spreadObject



185
186
# File 'app/models/tda/option.rb', line 185

def self.create_long_debit_call_spread
end

.create_short_debit_put_spreadObject



187
188
# File 'app/models/tda/option.rb', line 187

def self.create_short_debit_put_spread
end

.get_chain(params) ⇒ Object

2023-02-05 vp

Gets the entire chain



13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
# File 'app/models/tda/option.rb', line 13

def self.get_chain params
  opts = { symbol: params[:ticker] } ## use 'GME' as symbol here even though a symbol is eg 'GME_021023P2.5'
  query = { apikey: ::TD_AMERITRADE[:apiKey] }.merge opts
  puts! query, 'input opts'

  path = "/v1/marketdata/chains"
  out = self.get path, { query: query }
  timestamp = DateTime.parse out.headers['date']
  out = out.parsed_response.deep_symbolize_keys


  outs = []
  %w| put call |.each do |contractType|
    tmp_sym = "#{contractType}ExpDateMap".to_sym
    _out = out[tmp_sym]
    _out.each do |date, vs| ## date="2023-02-10:5"
      vs.each do |strike, _v| ## strike="18.5"
        v = _v[0] ## v={} many attrs
        v = v.except( :lastSize, :optionDeliverablesList, :settlementType,
          :deliverableNote, :pennyPilot, :mini )
        v.each do |k, i|
          if i == 'NaN'
            v[k] = nil
          end
        end

        v[:timestamp] = timestamp
        v[:ticker] = params[:ticker]
        outs.push( v )
      end
    end
  end

  outs.each do |x|
    opi = ::Iro::OptionPriceItem.create( x )
    if !opi.persisted?
      puts! opi.errors.full_messages, "Cannot create OptionPriceItem"
    end
  end
end

.get_quote(params) ⇒ Object

2023-03-18 vp This is what I should be using to check if a position should be rolled.



57
58
59
# File 'app/models/tda/option.rb', line 57

def self.get_quote params
  OpenStruct.new ::Tda::Option.get_quotes(params)[0]
end

.get_quotes(params) ⇒ Object

params: contractType, strike, expirationDate, ticker

ow = { contractType: ‘PUT’, ticker: ‘GME’, date: ‘2022-12-09’ } query = :toDate=>“2022-12-09”, :fromDate=>“2022-12-09”, :symbol=>“GME”

2023-02-04 vp

Too specific, but I want the entire chain, every 1-min

2023-02-06 vp

Continue.



70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
# File 'app/models/tda/option.rb', line 70

def self.get_quotes params
  puts! params, 'Tda::Option#get_quotes'
  opts = {}

  #
  # Validate input ???
  #
  validOpts = %i| contractType |
  validOpts.each do |s|
    if params[s]
      opts[s] = params[s]
    else
      raise Iro::InputError.new("Invalid input, missing '#{s}'.")
    end
  end
  if params[:expirationDate]
    opts[:fromDate] = opts[:toDate] = params[:expirationDate]
  else
    raise Iro::InputError.new("Invalid input, missing 'date'.")
  end
  if params[:ticker]
    opts[:symbol] = params[:ticker].upcase
  else
    raise Iro::InputError.new("Invalid input, missing 'ticker'.")
  end

  if params[:strike]
    opts[:strike] = params[:strike]
  end

  query = { apikey: ::TD_AMERITRADE[:apiKey] }.merge opts
  # puts! query, 'input opts'

  path = "/v1/marketdata/chains"
  out = self.get path, { query: query }
  timestamp = DateTime.parse out.headers['date']
  ## out = HTTParty.get "https://api.tdameritrade.com#{path}", { query: query }
  out = out.parsed_response.deep_symbolize_keys


  tmp_sym = "#{opts[:contractType].to_s.downcase}ExpDateMap".to_sym
  outs = []
  out = out[tmp_sym]
  out.each do |date, vs|
    vs.each do |strike, _v|
      v = _v[0]
      v = v.except( :lastSize, :optionDeliverablesList, :settlementType,
        :deliverableNote, :pennyPilot, :mini )
      v[:timestamp] = timestamp
      outs.push( v )
    end
  end

  # puts! outs, 'outs'
  return outs
end

.get_tokenObject



135
136
137
138
139
140
141
# File 'app/models/tda/option.rb', line 135

def self.get_token
  opts = {
    grant_type: 'authorization_code',
    access_type: 'offline',
    code: ::TD_AMERITRADE[:code],
  }
end

.roll_credit_callObject



190
191
# File 'app/models/tda/option.rb', line 190

def self.roll_credit_call
end

.roll_long_debit_call_spreadObject



192
193
# File 'app/models/tda/option.rb', line 192

def self.roll_long_debit_call_spread
end

.roll_short_debit_put_spreadObject



194
195
# File 'app/models/tda/option.rb', line 194

def self.roll_short_debit_put_spread
end